/*** PeriodicallySwitchController Z-Way HA module *******************************************

Version: 1.0.2
(c) Z-Wave.Me, 2015
-----------------------------------------------------------------------------
Author: Niels Roche <nir@zwave.eu>
Description:
    Controlls your switches during a configured time periode. 
    After the time periode, a selected switch is again set to On / Off or the last selected level.
******************************************************************************/

// ----------------------------------------------------------------------------
// --- Class definition, inheritance and setup
// ----------------------------------------------------------------------------

function PeriodicallySwitchController (id, controller) {
    // Call superconstructor first (AutomationModule)
    PeriodicallySwitchController.super_.call(this, id, controller);
}

inherits(PeriodicallySwitchController, AutomationModule);

_module = PeriodicallySwitchController;

// ----------------------------------------------------------------------------
// --- Module instance initialized
// ----------------------------------------------------------------------------

PeriodicallySwitchController.prototype.init = function (config) {
    PeriodicallySwitchController.super_.prototype.init.call(this, config);

 	var self = this;

    //change status of switches
    this.switchAction = function() {
        if(self.config.switches.length > 0){
            self.config.switches.forEach(function (singleSwitch) {
                
                var switchType = singleSwitch.filter,
                    curSwitch = singleSwitch[switchType],
                    vDev = self.controller.devices.get(curSwitch.device);

                // switch binary switches
                if (vDev && switchType === 'switchBinary') {
                    vDev.performCommand(curSwitch.status);

                // switch multilevel switches
                } else if (vDev && switchType === 'switchMultilevel'){
                    if (curSwitch.status === 'lvl') {
                        curSwitch.lastLevel = vDev.get('metrics:level');
                        vDev.performCommand("exact", { level: curSwitch.level });
                    } else {
                        vDev.performCommand(curSwitch.status);
                    } 
                }
            });
        }
    };

    //revert status of switches
    this.revertSwitchAction = function() {
        if(self.config.switches.length > 0){
            self.config.switches.forEach(function (singleSwitch) {
                
                var switchType = singleSwitch.filter,
                    curSwitch = singleSwitch[switchType],
                    vDev = self.controller.devices.get(curSwitch.device);

                // revert binary switches
                if (vDev && switchType === 'switchBinary') {
                    if(curSwitch.status === 'off'){
                        vDev.performCommand('on');
                    } else {
                        vDev.performCommand('off');
                    }                    

                // revert multilevel switches
                } else if (vDev && switchType === 'switchMultilevel'){
                    if (curSwitch.lastLevel && curSwitch.status === 'lvl') {
                        // switch back to level before
                        vDev.performCommand("exact", { level: curSwitch.lastLevel });
                    } else if (curSwitch.status === 'off'){
                        vDev.performCommand('on');
                    } else {
                        vDev.performCommand('off');
                    }
                }
            });
        }
    };

    var tFArray = self.config.periodOfTime.from.split(":");
    var mF = parseInt(tFArray[1], 10);
    var hF = parseInt(tFArray[0], 10);

    // add cron schedule for 'from'
    this.controller.emit("cron.addTask", "fromActionOnSwitches.poll."+self.id, {
        minute: mF,
        hour: hF,
        weekDay: null,
        day: null,
        month: null
    });

    var tTArray = self.config.periodOfTime.to.split(":");
    var mT = parseInt(tTArray[1], 10);
    var hT = parseInt(tTArray[0], 10);

    // add cron schedule for 'to'
    this.controller.emit("cron.addTask", "toActionOnSwitches.poll."+self.id, {
        minute: mT,
        hour: hT,
        weekDay: null,
        day: null,
        month: null
    });

    // set up cron handler
    this.controller.on("fromActionOnSwitches.poll."+self.id, this.switchAction);
    this.controller.on("toActionOnSwitches.poll."+self.id, this.revertSwitchAction);
};

PeriodicallySwitchController.prototype.stop = function () {

    var self = this;

    // remove cron schedule for 'from'
    this.controller.emit("cron.removeTask", "fromActionOnSwitches.poll."+self.id);
    this.controller.off("fromActionOnSwitches.poll."+self.id, self.switchAction);

    // remove cron schedule for 'to'
    this.controller.emit("cron.removeTask", "toActionOnSwitches.poll."+self.id);
    this.controller.off("toActionOnSwitches.poll."+self.id, self.revertSwitchAction);

    PeriodicallySwitchController.super_.prototype.stop.call(this);
};

// ----------------------------------------------------------------------------
// --- Module methods
// ----------------------------------------------------------------------------